/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;

@Entity
@Table(name = "project_measures")
public class ProjectMeasure {

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "PROJECT_MEASURES_SEQ", sequenceName = "PROJECT_MEASURES_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "PROJECT_MEASURES_SEQ")
  private Integer id;

  @Column(name = "value", updatable = false, nullable = false, precision = 30, scale = 20)
  private Double value = 0.0;

  @Column(name = "text_value", updatable = false, nullable = true, length = 96)
  private String textValue;

  @Column(name = "tendency", updatable = true, nullable = true)
  private Integer tendency;

  @ManyToOne(fetch = FetchType.EAGER)
  @JoinColumn(name = "metric_id")
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  private Metric metric;

  @Column(name = "snapshot_id")
  private Integer snapshotId;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "rule_id")
  @Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
  private Rule rule;

  @Column(name = "rules_category_id")
  private Integer rulesCategoryId;


  public ProjectMeasure(Snapshot snapshot, MeasureKey key, Double value) {
    this(snapshot, key.getMetric(), value);
    rule = key.getRule();
    rulesCategoryId = key.getRulesCategoryId();
  }

  public ProjectMeasure(Snapshot snapshot, Metric metric, Double value) throws IllegalArgumentException {
    this.snapshot = snapshot;
    if (snapshot != null) {
      this.snapshotId = snapshot.getId();
    }
    this.metric = metric;
    if (value != null && (value.isNaN() || value.isInfinite())){
      throw new IllegalArgumentException();
    }
    this.value = value;
  }

  public ProjectMeasure(Snapshot snapshot, Metric metric, String value) {
    this(snapshot, metric, (Double) null);
    this.value = 0.0;
    this.textValue = value;
  }

  public Double getValue() {
    return value;
  }

  public void setValue(Double value) throws IllegalArgumentException {
    if (value != null && (value.isNaN() || value.isInfinite())){
      throw new IllegalArgumentException();
    }
    this.value = value;
  }

  public String getTextValue() {
    return textValue;
  }

  public void setTextValue(String textValue) {
    this.textValue = textValue;
  }

  public Integer getTendency() {
    return tendency;
  }

  public void setTendency(Integer tendency) {
    this.tendency = tendency;
  }

  public Metric getMetric() {
    return metric;
  }

  public void setMetric(Metric metric) {
    this.metric = metric;
  }

  public Integer getSnapshotId() {
    return snapshotId;
  }

  public void setSnapshotId(Integer snapshotId) {
    this.snapshotId = snapshotId;
  }

  @Transient
  private Snapshot snapshot = null;
  public Snapshot ephemereSnapshot() {
    return snapshot;
  }

  public Rule getRule() {
    return rule;
  }

  public void setRule(Rule rule) {
    this.rule = rule;
  }

  public Integer getRulesCategoryId() {
    return rulesCategoryId;
  }

  public void setRulesCategoryId(Integer id) {
    this.rulesCategoryId = id;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public ProjectMeasure() {
  }

  @Override
  public String toString() {
    return new ToStringBuilder(this).
        append("value", value).
        append("metric", metric).
        toString();
  }

  public Integer getRuleId() {
    if (getRule()!=null) {
      return getRule().getId();
    }
    return null;
  }
}
