/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons;

import ch.hortis.sonar.service.MeasuresDao;
import ch.hortis.sonar.service.PropertiesDao;
import org.sonar.commons.database.BatchDatabaseManager;
import org.sonar.commons.database.DatabaseConnector;
import org.sonar.commons.database.DatabaseManager;
import org.sonar.commons.rules.RulesDao;

public class DaoFacade {

  private DatabaseManager databaseManager = null;
  private RulesDao rulesDao = null;
  private MeasuresDao measuresDao = null;
  private PropertiesDao propertiesDao = null;


  public DaoFacade(DatabaseConnector databaseConnector) {
    databaseManager = new BatchDatabaseManager(databaseConnector);
    databaseManager.start();
  }

  public void start() {
  }

  public void stop() {
    if (databaseManager != null) {
      databaseManager.stop();
      rulesDao = null;
      measuresDao = null;
    }
  }

  public RulesDao getRulesDao() {
    if (rulesDao == null) {
      rulesDao = new RulesDao(databaseManager);
    }
    return rulesDao;
  }

  public MeasuresDao getMeasuresDao() {
    if (measuresDao == null) {
      measuresDao = new MeasuresDao(getDatabaseManager());
    }
    return measuresDao;
  }

  public PropertiesDao getPropertiesDao() {
    if (propertiesDao == null) {
      propertiesDao = new PropertiesDao(getDatabaseManager().getEntityManager());
    }
    return propertiesDao;
  }

  public DatabaseManager getDatabaseManager() {
    return databaseManager;
  }

  protected void finalize() throws Throwable {
    stop();
    super.finalize();
  }
}
