/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.commands;

import java.util.concurrent.*;

public class CommandAsynchronousExecutor {

    private int nbTaskExecutors = 1;
    private long timeout = 0;

    private static BlockingQueue<Runnable> queue = new LinkedBlockingQueue<Runnable>();
    private ThreadPoolExecutor threadPool = null;
    private CommandExceptionNotifier notifier = null;

    // ------------------------------------------------------------------------
    // -- Constructors
    // ------------------------------------------------------------------------
    public CommandAsynchronousExecutor() {
    }

    public CommandAsynchronousExecutor(int nbTaskExecutors) {
        this.nbTaskExecutors = nbTaskExecutors;
    }

    public CommandAsynchronousExecutor(int nbTaskExecutors, CommandExceptionNotifier notifier) {
        this.nbTaskExecutors = nbTaskExecutors;
        this.notifier = notifier;
    }

    // ------------------------------------------------------------------------
    // -- Public methods
    // ------------------------------------------------------------------------
    public void runTask(CommandTask task) {
        task.setTimeout(timeout);
        task.setNotifier(notifier);
        if(threadPool == null) {
            threadPool = new ThreadPoolExecutor(nbTaskExecutors, nbTaskExecutors, Long.MAX_VALUE, TimeUnit.MICROSECONDS, queue);
        }
        threadPool.execute(task);
    }

    public void runTask(String... args) {
        CommandTask task = new CommandTask(args);
        runTask(task);
    }

    public void shutDown() {
        threadPool.shutdown();
    }

    // ------------------------------------------------------------------------
    // -- Getters and setters
    // ------------------------------------------------------------------------
    public ThreadPoolExecutor getThreadPool() {
        return threadPool;
    }
    public void setThreadPool(ThreadPoolExecutor threadPool) {
        this.threadPool = threadPool;
    }
    public long getTimeout() {
        return timeout;
    }
    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }

}
