/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.metrics;

import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.model.MetricType;

import java.util.ArrayList;
import java.util.List;
import java.lang.reflect.Field;

public class CoreMetrics {
  public static final String DOMAIN_TESTS = "Tests";


  /* core statistics */
  public static final Metric NCSS = new Metric("ncss", "Non Commenting Source Statements", MetricType.INT, -1, null, "NCSS", "Non Commenting Source Statements", false, true);

  public static final Metric CLASSES_COUNT = new Metric("classes_count", "Classes", MetricType.INT, -1, null, "Classes", "Nb of classes", false, true);

  public static final Metric PACKAGES_COUNT = new Metric("packages_count", "Packages", MetricType.INT, -1, null, "Packages", "Nb of Packages", false, true);

  public static final Metric FUNCTIONS_COUNT = new Metric("functions_count", "Methods", MetricType.INT, -1, null, "Methods", "Nb of methods", false, true);

  public static final Metric FILES_COUNT = new Metric("files_count", "Files", MetricType.INT, -1, null, "Files", "Nb of files", false, true);

  public static final Metric DIRECTORIES_COUNT = new Metric("directories_count", "Directories", MetricType.INT, -1, null, "Directories", "Nb of directories", false, true);

  /* complexity */
  public static final Metric COMPLEXITY_TOTAL = new Metric("ccn", "Cyclomatic complexity", MetricType.INT, -1, "Complexity", "Total complexity", "Total complexity", false, true);

  public static final Metric COMPLEXITY_AVG_BY_CLASS = new Metric("ccn_class", "Complexity, average per class", MetricType.FLOAT, -1, "Complexity", "Complexity by class", "Complexity by class", true, true);

  public static final Metric COMPLEXITY_AVG_BY_FUNCTION = new Metric("ccn_function", "Complexity, average per function", MetricType.FLOAT, -1, "Complexity", "Complexity by function", "Complexity by function", true, true);

  public static final Metric COMPLEXITY_AVG_BY_FILE = new Metric("ccn_file", "Complexity, average per file", MetricType.FLOAT, -1, "Complexity", "Complexity by file", "Complexity by file", true, true);

  public static final Metric COMPLEXITY_CLASSES_COUNT_DISTRIBUTION = new Metric("ccn_classes_count_distribution", "ccn_classes_count_distribution", MetricType.INT, -1, "Complexity", "ccn_classes_count_distribution", "ccn_classes_count_distribution", false, false);

  public static final Metric COMPLEXITY_CLASSES_PERCENT_DISTRIBUTION = new Metric("ccn_classes_percent_distribution", "ccn_classes_percent_distribution", MetricType.INT, -1, "Complexity", "ccn_classes_percent_distribution", "ccn_classes_percent_distribution", false, false);

  /* comments */
  public static final Metric COMMENT_LINES = new Metric("comment_lines", "Comment lines", MetricType.INT, 1, null, "Comment lines", "Comment lines", false, false);

  public static final Metric COMMENT_RATIO = new Metric("comment_ratio", "Comments", MetricType.PERCENTAGE, 1, null, "Comments", "Comments", true, false);

  /* unit tests */
  public static final Metric TESTS_COUNT = new Metric("test_count", "Test count", MetricType.INT, -1, null, "Test count", "Test count", false, false);

  public static final Metric TESTS_TIME = new Metric("test_execution_time", "Test execution time", MetricType.INT, -1, null, "Test execution time", "Test execution time", false, false);

  public static final Metric TESTS_ERRORS = new Metric("test_errors_count", "Test errors count", MetricType.INT, -1, null, "Test errors count", "Test errors count", false, false);

  public static final Metric TESTS_SKIPPED = new Metric("test_skipped_count", "Test skipped count", MetricType.INT, -1, null, "Test skipped count", "Test skipped count", false, false);

  public static final Metric TESTS_FAILURES = new Metric("test_failures_count", "Test failures count", MetricType.INT, -1, null, "Test failures count", "Test failures count", false, false);

  public static final Metric TESTS_SUCCESS_PERCENTAGE = new Metric("test_success_percentage", "Test success percentage", MetricType.INT, -1, null, "Test success percentage", "Test success percentage", false, false);

  public static final Metric TESTS_ERRORS_PERCENTAGE = new Metric("test_errors_percentage", "Test errors percentage", MetricType.INT, -1, null, "Test errors percentage", "Test errors percentage", false, false);

  public static final Metric TESTS_FAILURE_PERCENTAGE = new Metric("test_failure_percentage", "Test failure percentage", MetricType.INT, -1, null, "Test failure percentage", "Test failure percentage", false, false);

  public static final Metric CODE_COVERAGE = new Metric("code_coverage", "Code coverage", MetricType.PERCENTAGE, 1, DOMAIN_TESTS, "Code coverage", "Code coverage", true, true);

  /* duplicated lines */
  public static final Metric DUPLICATED_LINES = new Metric("duplicated_lines", "Duplicated lines", MetricType.INT, -1, null, "Duplicated lines", "Duplicated lines", false, false);

  public static final Metric DUPLICATED_LINES_RATIO = new Metric("duplicated_lines_ratio", "Duplicated lines ratio", MetricType.PERCENTAGE, -1, null, "Duplicated lines ratio", "Duplicated lines ratio", false, false);

  /* coding rules */
  public static final Metric RULES_MANDATORY_COUNT = new Metric("rules_violations", "rules_violations", MetricType.INT, -1, null, "rules_violations", "rules_violations", false, false);

  public static final Metric RULES_MANDATORY_INDEX = new Metric("rules_compliance", "rules_compliance", MetricType.INT, -1, null, "rules_compliance", "rules_compliance", false, false);

  public static final Metric RULES_OPTIONAL_COUNT = new Metric("optional_rules_violations", "optional_rules_violations", MetricType.INT, -1, null, "optional_rules_violations", "optional_rules_violations", false, false);

  public static final Metric RULES_OPTIONAL_INDEX = new Metric("optional_rules_compliance", "optional_rules_compliance", MetricType.INT, -1, null, "optional_rules_compliance", "optional_rules_compliance", false, false);

  public static final Metric RULES_ALL_INDEX = new Metric("strict_rules_compliance", "strict_rules_compliance", MetricType.INT, -1, null, "strict_rules_compliance", "strict_rules_compliance", false, false);

  private static List<Metric> metrics = new ArrayList<Metric>();

  public static final List<Metric> getMetrics() {
    if (metrics.isEmpty()) {
      for (Field field : CoreMetrics.class.getFields()) {
        if (Metric.class.isAssignableFrom(field.getType())) {
          try {
            metrics.add((Metric) field.get(null));
          } catch (IllegalAccessException e) {
            throw new RuntimeException("can not load metrics from CoreMetrics.class", e);
          }
        }
      }
    }
    return metrics;
  }
}
